require 'active_record/schema_migration'
require 'cell/meta'
require 'shellwords'

module Cell
  module CloneSchema
    def self.connection
      ::ActiveRecord::Base.connection
    end

    def self.configuration
      Rails.configuration.database_configuration[Rails.env]
    end

    def self.postgres_cli_args(cmd)
      fail ArgumentError unless [:psql, :pg_dump].include?(cmd)

      conf = configuration

      r = []
      r << '--host'     << conf['host'] if conf['host']
      r << '--port'     << conf['port'] if conf['port']
      r << '--dbname'   << conf['database'] if conf['database']
      r << '--username' << conf['username'] if conf['username']
      r << '--password' << conf['password'] if conf['password']
      # psql doesn't accept --encoding
      r << '--encoding' << conf['encoding'] if conf['encoding'] unless cmd == :psql

      r
    end

    def self.psql_cmd
      cmd = ["psql"] + postgres_cli_args(:psql).map{|s| Shellwords.escape(s)}
      cmd.join(" ")
    end

    def self.pg_dump_cmd(schema_name)
      cmd = ["pg_dump"] + postgres_cli_args(:pg_dump).map{|s| Shellwords.escape(s)}
      cmd << "--schema" << Shellwords.escape(schema_name)
      cmd.join(" ")
    end

    def self.filter_cmd(src, dst)
      # This is slightly dumb, but better than trying to do it in DDL
      "sed -e 's/#{src}/#{dst}/g'"
    end

    def self.clone_schema(src, dst)
      clone_cmd = "#{pg_dump_cmd(src)} | #{filter_cmd(src, dst)} | #{psql_cmd}"
      result = %x(#{clone_cmd})
      unless $?.success?
        fail ActiveRecord::StatementInvalid, result
      end
    end

    def self.copy_schema_migrations_to(dst)
      Cell::Meta.with_schema(dst, exclusive: true) do
        ::ActiveRecord::SchemaMigration.create_table
      end

      connection.execute <<~SQL
        INSERT INTO #{connection.quote_schema_name(dst)}.schema_migrations
          SELECT * from schema_migrations
      SQL
    end
  end
end
