require 'cell/schema'
require 'active_record/internal_metadata'

module Cell
  module Meta
    GLOBAL_KEY = 'Cell.global'

    def self.connection
      ::ActiveRecord::Base.connection
    end

    def self.with_schema(new_path, exclusive: false)
      saved_path = connection.schema_search_path
      active_path = exclusive ? new_path : "#{new_path},#{saved_path}"

      set_schema_search_path(active_path)

      if block_given?
        begin
          yield
        ensure
          set_schema_search_path(saved_path)
        end
      end
    end

    def self.set_schema_search_path(path)
      if path != connection.schema_search_path
        connection.schema_search_path = path
        connection.clear_query_cache
      end
    end

    def self.global_schema
      # This could be racy?
      @global_schema ||= ::ActiveRecord::Base.connection.schema_search_path
    end

    def self.prototype_schema
      'cell_prototype'
    end

    def self.structural_schema
      "#{prototype_schema},#{global_schema}"
    end

    def self.with_global_schema(&block)
      with_schema(global_schema, exclusive: true, &block)
    end

    def self.with_prototype_schema(&block)
      with_schema(prototype_schema, exclusive: true, &block)
    end

    def self.with_structural_schema(&block)
      with_schema(structural_schema, exclusive: true, &block)
    end

    def self.global_tables
      with_global_schema do
        (::ActiveRecord::InternalMetadata[GLOBAL_KEY] || '').split(',')
      end
    end

    def self.set_global_tables(tables)
      with_global_schema do
        table_string = tables.map(&:to_s).sort.uniq.join(',')
        ::ActiveRecord::InternalMetadata[GLOBAL_KEY] = table_string
      end
    end

    def self.add_global_table(name)
      set_global_tables(global_tables + [name.to_s])
    end

    def self.remove_global_table(name)
      set_global_tables(global_tables - [name.to_s])
    end

    def self.global_model?(m)
      @model_map ||= global_tables.map do |k|
        [k, true]
      end.to_h

      @model_map[m.table_name] || false
    end
  end
end
