require 'shellwords'
require 'cell/clone_schema'
require 'active_record/schema_migration'

module Cell
  module Schema
    SCHEMA_PREFIX = 't_'
    MAX_SCHEMA_NAME_LENGTH = 63 # PostgreSQL baked-in default
    MAX_CELL_ID_SIZE = MAX_SCHEMA_NAME_LENGTH - SCHEMA_PREFIX.size

    def self.schema_name_for_cell_id(cell_id)
      SCHEMA_PREFIX + cell_id.to_s.gsub(/[^a-z0-9_]/i, '-')
    end

    def schema_name
      Schema.schema_name_for_cell_id(cell_id)
    end

    private
    def create_schema!
      con = self.class.connection
      con.transaction do
        CloneSchema.clone_schema(Meta.prototype_schema, schema_name)
        CloneSchema.copy_schema_migrations_to(schema_name)
      end
    end

    def destroy_schema!
      con = self.class.connection
      con.execute "DROP SCHEMA #{con.quote_schema_name(schema_name)} CASCADE"
    end

    def update_schema!
      src, dst = cell_id_change_set
      src = self.class.schema_name_for_cell_id(src)

      con = self.class.connection
      con.execute "ALTER SCHEMA #{con.quote_schema_name(src)} RENAME " +
                  "TO #{con.quote_schema_name(schema_name)}"
    end

    def self.prepended(cls)
      cls.after_create :create_schema!
      cls.after_update_commit  :update_schema!, if: :cell_id_changed?
      cls.after_destroy_commit :destroy_schema!
    end
  end
end
