require 'cell/context'
require 'cell/schema'
require 'cell/url_options'

module Cell
  module Tenant
    module ClassMethods
      def cell_id_column
        :id
      end

      def cell_find(cell_id, finder_method: :find_by)
        send(finder_method, cell_id_column => cell_id)
      end

      def cell_find!(cell_id)
        cell_find(cell_id, finder_method: :find_by!)
      end
    end

    def cell_id
      send(self.class.cell_id_column)
    end

    def cell_id_changed?
      !! previous_changes[self.class.cell_id_column]
    end

    def cell_id_change_set
      fail "cell_id was not changed" unless cell_id_changed?
      previous_changes[self.class.cell_id_column]
    end

    def self.append_features(cls)
      cls.prepend(self)
    end

    def self.extend_object(cls)
      cls.prepend(self)
    end

    def self.prepended(model)
      Cell.assign_model(model)

      model.extend(ClassMethods)
      model.prepend(::Cell::Schema)
      model.prepend(::Cell::Context)
      model.prepend(::Cell::UrlOptions)
    end
  end

  def self.assign_model(model)
    remove_const(:Model) if const_defined?(:Model)
    const_set(:Model, model)
  end
end
